# Import statements
from requests_html import HTMLSession
import requests
import warnings
from validators.domain import domain
import sys

class NewsFinder():

    """
    API Object to scrape news articles from Google News and scrape the article text from the news website. 
    Allows filtering of Google News results.

    Note: Due to issues with scraping, Daily Mail and News18 are blocked by default. You can unblock them by calling the remove_blocked_source method but this is not advised.

    Attributes
    ----------
    blocked_sources : list
        A list of sources to block, by default includes Daily Mail and News18
    """

    #####################################
    # Initialization
    #####################################

    def __init__(self, blocked_sources: list|None = None):
        """Initialize the NewsFinder class
        
        Parameters
        ----------
        blocked_sources : list, optional
            A list of sources to block, by default includes Daily Mail and News18        

        Raises
        ------
        TypeError
            If blocked_sources is not a list
        TypeError
            If any element in blocked_sources is not a string    
        """

        if blocked_sources is None:
            self.blocked_sources = ["Daily Mail", "News18"]
        else:

            # Check if blocked_sources is valid
            if not isinstance(blocked_sources, list):
                raise TypeError("blocked_sources must be a list")
            
            # Check if all elements in blocked_sources are strings
            if not all(isinstance(source, str) for source in blocked_sources):
                raise TypeError("blocked_sources must be a list of strings")

            self.blocked_sources = blocked_sources + ["Daily Mail", "News18"]

    #####################################
    # Private Methods
    #####################################

    def __scrape_articles(self, url: str, number_of_articles: int = 5) -> list | None:
        """Private: Scrape the news articles from Google News for a given topic

        Parameters
        ----------
        url : str
            The google news url to scrape
        number_of_articles : int, optional 
            The number of articles to scrape, by default 5

        Returns
        -------
        all_articles : list
            A list of dictionaries containing the title, source, time and link of each article
        """

        # Initialize an HTML Session
        session = HTMLSession()

        # Get the page
        r = session.get(url=url)

        # Get all the articles
        try:
            articles = r.html.find('article')
        except:
            return None
        
        all_articles = []

        # Iterate over each article
        for article in articles:
            
            # Break if we have enough articles
            if len(all_articles) == number_of_articles:
                break

            # Get the title
            title = article.find('h3', first=True).text

            # Get the source
            source = article.find('img', first=True).attrs.get('alt')

            # Disallow certain sources
            if source in self.blocked_sources:
                continue
        
            # Get the link
            link = article.find('a', first=True).absolute_links.pop()

            # Print the details
            newsarticle = {
                'title': title,
                'source': source,
                'link': link
            }
            all_articles.append(newsarticle)

        return all_articles
    
    def __scrape_news_article(self, url: str) -> dict | None:
        """Private: Scrape the news article from the given url

        Parameters
        ----------
        url : str
            The google news url of the news article
        
        Returns
        -------
        article : dict
            A dictionary containing the title and article body of the news article
        """


        # Final url
        try:
            url = requests.get(url, timeout=5).url
        except Exception as error:
            warnings.warn(f"Error processing url: {url}. Continuing without it...")
            return None

        # Initialize HTML Session
        session = HTMLSession()

        # Get the page
        r = session.get(url=url)

        # Get the title
        try:
            title  = r.html.find('h1', first=True).text
        except:
            title = ""

        # Get all article fragments (each fragment is a paragraph)
        try:
            article_fragments = r.html.find('p')
        except:
            warnings.warn(f"Article with url: {url} cannot be scraped. Continuing without it...")
            return None


        # Join all the paragraphs to form the article
        body = '\n'.join([fragment.text for fragment in article_fragments])

        return {'title': title, 'article': body}
    
    def __build_list_of_articles(self, articles_list: list) -> list:
        """Private: Build a list of articles from the given list of dictionaries
        
        Parameters
        ----------
        articles_list : list(dict)
            A list of dictionaries containing the title, source, date and link of the articles

        Returns
        -------
        articles_full_text : list
            A list of dictionaries containing the title and article body of the news articles
        """

        # Iterate through articles, and scrape each one
        for article in articles_list:
            article_text = self.__build_article_from_dict(article)
            article.update(article_text)
        return articles_list
    
    def __build_article_from_dict(self, article_dict: dict) -> dict | None:
        """Private: Build the article from the given dictionary

        Parameters
        ----------
        article_dict : dict
            A dictionary containing the title, source, date and link of the article

        Returns
        -------
        article : dict
            A dictionary containing the title and article body of the news article
        """

        return self.__scrape_news_article(article_dict['link'])
    
    def __build_url(self, topic_url: str, source_url: str, period_url: str) -> str:
        """Private: Build the url for the given topic, source and period
        
        Parameters
        ----------
        topic_url : str
            The topic to search for
        source_url : str
            The url fragment of the source to search for
        period_url : str
            The url fragment of the period to search for

        Returns 
        -------
        url : str
            The url to scrape
        """

        return f'https://news.google.com/search?q={topic_url}{source_url}{period_url}&hl=en-IN&gl=IN&ceid=IN:en'
        
    
    ###############################
    # Public Methods - API Methods
    ###############################

    def add_blocked_source(self, sources: list) -> int:
        """Add a list of sources to block

        Parameters
        ----------
        sources : list
            A list of sources to block

        Returns
        -------
        int
            The number of blocked sources

        Raises
        ------
        TypeError
            If sources is not a list

        Examples
        --------
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder() 
        >>> nf.add_blocked_source(["BBC"]) # Add BBC to the list of blocked sources
        3
        """

        # Check if the sources is valid
        if not isinstance(sources, list):
            raise TypeError("Sources must be a list")

        # Check if the source is already blocked
        for source in sources:
            if source in self.blocked_sources:
                sources.remove(source)

        self.blocked_sources = self.blocked_sources + sources
        return len(self.blocked_sources)
    
    def remove_blocked_source(self, sources: list) -> int:
        """Remove a list of sources to block

        Parameters
        ----------
        sources : list
            A list of sources to block

        Returns
        -------
        int
            The number of blocked sources
        
        Raises
        ------
        TypeError
            If sources is not a list
        
        Examples
        --------
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.remove_blocked_source(["Daily Mail"]) # Remove Daily Mail from the list of blocked sources
        1
        """

        # Check if the sources is valid
        if not isinstance(sources, list):
            raise TypeError("Sources must be a list")

        for source in sources:
            self.blocked_sources.remove(source)
        return len(self.blocked_sources)
    
    def get_blocked_sources(self) -> list:
        """Get the list of blocked sources

        Returns
        -------
        list
            The list of blocked sources
        
        Examples
        --------
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.get_blocked_sources() # Remember, Daily Mail and News18 are blocked by default (due to issues with scraping)
        ['Daily Mail', 'News18']
        """

        return self.blocked_sources
    
    @property
    def blocked_sources(self) -> list:
        """Get the list of blocked sources

        Returns
        -------
        list
            The list of blocked sources
        
        Examples
        --------
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.blocked_sources # Remember, Daily Mail and News18 are blocked by default (due to issues with scraping)
        ['Daily Mail', 'News18']
        """

        return self.__blocked_sources
    
    @blocked_sources.setter
    def blocked_sources(self, sources: list) -> None:
        """Set the list of blocked sources

        Parameters
        ----------
        sources : list
            A list of sources to block
            
        Raises
        ------
        TypeError
            If sources is not a list

        Examples
        --------
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.blocked_sources = ["BBC"] # Set the list of blocked sources to BBC
        """

        # Check if the sources is valid
        if not isinstance(sources, list):
            raise TypeError("sources must be a list")

        self.__blocked_sources = sources

    def update_blocked_sources(self, sources: list) -> int:
        """Update the list of blocked sources by completely replacing existing blocked sources

        Parameters
        ----------
        sources : list
            A list of sources to block

        Returns
        -------
        int
            The number of blocked sources
        
        Raises
        ------
        TypeError
            If sources is not a list

        Examples
        --------
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.update_blocked_sources(["BBC"]) # Update the list of blocked sources to BBC
        1
        """
        # Check if the sources is valid
        if not isinstance(sources, list):
            raise TypeError("Sources must be a list")
        
        self.blocked_sources = sources
        return len(self.blocked_sources)
    
    def get_news_articles(self, topic: str|None = None, number_of_articles: int|None = None, source: str|None = None, period: str = "Any time", article_text: bool = False) -> list:
        """Get the news articles for a given topic or for a given source filtered by date
        
        Parameters
        ----------
        topic : str, optional
            The topic to search for, by default None
        number_of_articles : int, optional
            The number of articles to scrape, by default None which gives all the possible articles
        source : str, optional
            The domain for the website of the source to search for, by default None. For example, "dailymail.co.uk" or "bbc.com"
        period : list, optional
            The period to search for, by default "Any time". Period must be one of ["Any time", "Past hour", "Past 24 hours", "Past week", "Past year"]
        article_text : bool, optional
            Whether to scrape the article text or not, by default False
        
        Returns
        -------
        list
            A list of dictionaries containing the title, source, link and article body of the news articles (only if article_text is True)

        Raises
        ------
        ValueError
            If the period is not one of ["Any time", "Past hour", "Past 24 hours", "Past week", "Past year"].
        ValueError
            If the number_of_articles is not a positive integer
        TypeError
            If the topic is not a string
        TypeError
            If the source is not a string
        ValueError
            If the source is not a valid domain name
        TypeError
            If the article_text is not a boolean
        ValueError
            If the topic and source are both None

        Examples
        --------

        Retrieving all articles for a given topic in the last 24 hours without scraping.

        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.get_news_articles(topic="Donald Trump", period="Past 24 hours")

        Retrieving all articles for a given topic in the last 24 hours and scraping the article text.

        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.get_news_articles(topic="Donald Trump", period="Past 24 hours", article_text=True)

        Retrieving 5 articles from a given source in the last week without scraping.

        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> nf = NewsFinder()
        >>> nf.get_news_articles(source="bbc.com", period="Past week", number_of_articles=5)
        """

        # Check if the period is valid
        if period not in ["Any time", "Past hour", "Past 24 hours", "Past week", "Past year"]:
            raise ValueError("period must be one of ['Any time', 'Past hour', 'Past 24 hours', 'Past week', 'Past year']")
        
        # Check if the number of articles is valid
        if number_of_articles is not None and number_of_articles <= 0:
            raise ValueError("number_of_articles must be a positive integer")
        
        # Check if the topic is valid
        if topic is not None and not isinstance(topic, str):
            raise TypeError("topic must be a string")
        
        # Check if the source is valid
        if source is not None and not isinstance(source, str):
            raise TypeError("source must be a string")

        if source is not None and not domain(source):
            raise ValueError("source must be a valid domain name")

        # Check if the article_text is valid
        if not isinstance(article_text, bool):
            raise TypeError("article_text must be a boolean")
        
        # Check if the topic and source are both None
        if topic is None and source is None:
            raise ValueError("Either or both topic and source must be provided")

        ################### Build url ######################

        # If topic is provided
        if topic is not None:
            topic_url = topic + " "
        else:
            topic_url = ""

        # If source is provided
        if source is not None:
            source_url = " site:" + source
        else:
            source_url = ""

        # If period is provided

        period_mappings = {"Any time": "",
                           "Past hour": " when:1h",
                           "Past 24 hours": " when:1d",
                           "Past week": " when:7d",
                           "Past year": " when:1y"}

        period_url = period_mappings[period]

        url = self.__build_url(topic_url, source_url, period_url)


        ################### Scrape ######################

        articles = self.__scrape_articles(url, number_of_articles)

        # Check and report if no articles found
        if articles is None:
            sys.stdout.write("No articles found. Try different parameters")
            return []
        
        
        ################### Build Articles ######################
        if article_text:
            articles = self.__build_list_of_articles(articles)

        return articles


import click
import os
from cron_validator import CronValidator

def __give_yaml_text(self, cron: str, file: str) -> str:
    """Private: Build the .yaml file text for Github Actions to run the cron job
    
    Parameters
    ----------
    cron : str
        The cron job to run the pipeline.
        You can get a formatted cron job string from https://crontab.guru/
    file : str
        The file to run the pipeline from

    Returns
    -------
    str
        The .yaml file text for Github Actions to run the cron job
    """
    
    # Build the .yaml file text
    text = f"""---
name: "Twitter News Bot"
on:
schedule:
- cron: '{cron}'

jobs:
python-job:
name: "Python job"
runs-on: ubuntu-latest
steps:
    - name: Checkout repository
    uses: actions/checkout@v2
    - name: Setup python
    uses: actions/setup-python@v2
    with:
        python-version: '3.11.3'
    - name: Install dependencies
    run: pip install requests==2.31.0 requests-html==0.10.0 tweepy==4.14.0 google-generativeai==0.1.0 validators==0.20.0 cron-validator==1.0.8 python-dotenv==1.0.0
    - name: Run python script
    run: python {file}"""

    return text

@click.group()
def cli():
    pass

@cli.command(name="build-yaml")
@click.argument("cron")
@click.argument("filename")
def build_yaml(cron: str, filename: str) -> None:
    """Build a .yaml file for Github Actions to run the cron job
    
    Parameters
    ----------
    cron : str
        The cron job to run the pipeline.
        You can get a formatted cron job string from https://crontab.guru/
    filename : str
        The file to run the pipeline from

    Returns
    -------
    None

    Raises
    ------
    TypeError
        If cron is not a string
    TypeError
        If file_name is not a string
    ValueError
        If cron is not a valid cron job
    """

    # Check if cron is a string
    if not isinstance(cron, str):
        raise TypeError("cron must be a string")
    
    # Check if file is a string
    if not isinstance(filename, str):
        raise TypeError("file_name must be a string")
    
    # Check if cron is valid cron job
    if CronValidator.parse(cron) is None:
        raise ValueError("cron must be a valid cron job")
    
    # Build the .yml file
    text = __give_yaml_text(cron=cron, file=filename)

    try:
        os.mkdir(".github")
    except FileExistsError:
        pass

    try:
        os.mkdir(".github/workflows")
    except FileExistsError:
        pass

    file = open(".github/workflows/python-app.yml", "w")
    file.write(text)
    file.close()

if __name__ == "__main__":
    cli()


# Import statements
from newsfinder import NewsFinder
from tweeter import Tweeter

class TwitterNewsBot():
    """
    API Object to connect the NewsFinder and Tweeter classes to build a pipeline to automate finding, scraping and tweeting news.

    Attributes
    ----------
    news_finder : NewsFinder
        The NewsFinder object to use to find and scrape news articles
    tweeter_obj : Tweeter
        The Tweeter object to use to tweet the news articles
    topic : str
        The topic to search for news articles
    no_of_articles : int
        The number of articles to find and scrape, by default 5    
    """

    #####################################
    # Initialization
    #####################################

    def __init__(self, news_finder: NewsFinder, tweeter_obj: Tweeter, topic: str, no_of_articles: int = 5):
        """Initialize the Bot class
        
        Parameters
        ----------
        news_finder : NewsFinder
            The NewsFinder object to use to find and scrape news articles
        tweeter_obj : Tweeter
            The Tweeter object to use to tweet the news articles
        topic : str
            The topic to search for news articles
        no_of_articles : int, optional
            The number of articles to find and scrape, by default 5

        Returns
        -------
        None

        Raises
        ------
        TypeError
            If news_finder is not a NewsFinder object
            If tweeter_obj is not a Tweeter object
            If topic is not a string
            If no_of_articles is not an integer
        """
        
        # Check if news_finder is a NewsFinder object
        if not isinstance(news_finder, NewsFinder):
            raise TypeError("news_finder must be a NewsFinder object")
        
        # Check if tweeter_obj is a Tweeter object
        if not isinstance(tweeter_obj, Tweeter):
            raise TypeError("tweeter_obj must be a Tweeter object")
        
        # Check if topic is a string
        if not isinstance(topic, str):
            raise TypeError("topic must be a string")
        
        # Check if no_of_articles is an integer
        if not isinstance(no_of_articles, int):
            raise TypeError("no_of_articles must be an integer")
        
        # Set the attributes
        self.news_finder = news_finder
        self.tweeter_obj = tweeter_obj
        self.__topic = topic
        self.__no_of_articles = no_of_articles

    #####################################
    # Private Methods
    #####################################
    
    def __build_pipeline(self, **kwargs) -> dict:
        """Private: Build the pipeline to find, scrape and tweet news articles

        Parameters
        ----------
        **kwargs : dict
            Arguments for Tweeter.tweet function excluding articles_list. See Tweeter.tweet for more details on the args. 
        
        Returns
        -------
        dict
            A dictionary containing the total character count, the number of tweets posted, and the id of the parent tweet when run
        """
        
        # Build the pipeline
        articles = self.news_finder.get_news_articles(topic=self.__topic,number_of_articles=self.__no_of_articles, article_text=True)
        return self.tweeter_obj.tweet(articles_list=articles, **kwargs)
    
    ###############################
    # Public Methods - API Methods
    ###############################

    @property
    def topic(self) -> str:
        """Returns the topic to search for news articles
        
        Returns
        -------
        str
            The topic to search for news articles

        Examples
        --------
        >>> from twitternewsbot.twitternewsbot import TwitterNewsBot
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> from twitternewsbot.tweeter import Tweeter
        >>> bot = TwitterNewsBot(NewsFinder(), Tweeter(), "Python")
        >>> bot.topic
        "Python"
        """
        return self.__topic

    @topic.setter
    def topic(self, topic: str) -> None:
        """Sets the topic to search for news articles
        
        Parameters
        ----------
        topic : str
            The topic to search for news articles

        Raises
        ------
        TypeError
            If topic is not a string

        Examples
        --------
        >>> from twitternewsbot.twitternewsbot import TwitterNewsBot
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> from twitternewsbot.tweeter import Tweeter
        >>> bot = TwitterNewsBot(NewsFinder(), Tweeter(), "Python")
        >>> bot.topic = "HTML"
        """
        # Check if topic is a string
        if not isinstance(topic, str):
            raise TypeError("topic must be a string")
        
        self.__topic = topic
        return self.__topic

    @property
    def no_of_articles(self) -> int:
        """Returns the number of articles to find and scrape
        
        Returns
        -------
        int
            The number of articles to find and scrape

        Examples
        --------
        >>> from twitternewsbot.twitternewsbot import TwitterNewsBot
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> from twitternewsbot.tweeter import Tweeter
        >>> bot = TwitterNewsBot(NewsFinder(), Tweeter(), "Python")
        >>> bot.no_of_articles
        5
        """
        return self.__no_of_articles
    
    @no_of_articles.setter
    def no_of_articles(self, no_of_articles: int) -> None:
        """Sets the number of articles to find and scrape
        
        Parameters
        ----------
        no_of_articles : int
            The number of articles to find and scrape

        Returns
        -------
        None

        Raises
        ------
        TypeError
            If no_of_articles is not an integer

        Examples
        --------
        >>> from twitternewsbot.twitternewsbot import TwitterNewsBot
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> from twitternewsbot.tweeter import Tweeter
        >>> bot = TwitterNewsBot(NewsFinder(), Tweeter(), "Python")
        >>> bot.no_of_articles = 10
        """
        # Check if no_of_articles is an integer
        if not isinstance(no_of_articles, int):
            raise TypeError("no_of_articles must be an integer")
        
        self.__no_of_articles = no_of_articles

    def run(self, **kwargs) -> dict:
        """
        Build article list, scrape articles and tweet summarized tweet for the given topic.

        Parameters
        ----------
        **kwargs : dict
            Arguments for Tweeter.tweet function excluding articles_list. See Tweeter.tweet for more details on the args.

        Returns
        -------
        dict
            A dictionary containing the total character count, the number of tweets posted, and the id of the parent tweet when run
        
        Examples
        --------

        Creating a basic pipeline to find, scrape and tweet news articles:

        >>> from twitternewsbot.twitternewsbot import TwitterNewsBot
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> from twitternewsbot.tweeter import Tweeter
        >>> bot = TwitterNewsBot(NewsFinder(), Tweeter(), "Python")
        >>> bot.run()
        """

        # Run the pipeline
        return self.__build_pipeline(**kwargs)



import google.generativeai as palm
from tweepy import Client
import os
from dotenv import load_dotenv

class Tweeter():

    """
    API Object allowing users to tweet articles, summaries and other text to Twitter. 
    It leverages PaLM to summarize articles and then tweets them in chunks of 280 characters.
    The API keys required for the Twitter API must be provided as environment variables or as arguments to the constructor.
    The API key required for the Google API can be provided as environment variables or as an argument to the constructor. 
    The Google API key is optional. If not provided, the API will not be able to summarize articles.

    Attributes
    ----------
    __API_KEY : str
        The API key for the Twitter API
    __API_SECRET_KEY : str
        The API secret key for the Twitter API
    __ACCESS_TOKEN : str
        The access token for the Twitter API
    __ACCESS_TOKEN_SECRET : str
        The access token secret for the Twitter API
    __GOOGLE_API_KEY : str
        The API key for the Google API
    __client : Client
        The tweepy client object
    """

  #####################################
  # Initialization
  #####################################
    
    def __init__(self, api_key: str|None = None, api_secret_key: str|None = None, access_token: str|None = None, access_token_secret: str|None = None, google_api_key: str|None = None):
        """Initialize the class with tokens and tweepy client
        
        Parameters
        ----------
        api_key : str, optional
            The API key for the Twitter API, by default None
        api_secret_key : str, optional
            The API secret key for the Twitter API, by default None
        access_token : str, optional
            The access token for the Twitter API, by default None
        access_token_secret : str, optional
            The access token secret for the Twitter API, by default None
        google_api_key : str, optional
            The API key for the Google API, by default None

        Raises
        ------
        Exception
            If API_KEY is not found in environment variables and not provided as an argument
        Exception
            If API_SECRET_KEY is not found in environment variables and not provided as an argument
        Exception
            If ACCESS_TOKEN is not found in environment variables and not provided as an argument
        Exception
            If ACCESS_TOKEN_SECRET is not found in environment variables and not provided as an argument
        Exception
            Twitter API Authentication Failed. Invalid Twitter API Credentials
        """

        # Load environment variables
        load_dotenv()

        if api_key is None:
            try:
                self.__API_KEY = os.getenv("API_KEY")
            except:
                raise Exception("API_KEY not found in environment variables and not provided as an argument")
        else:
            self.__API_KEY = api_key
        
        if api_secret_key is None:
            try:
                self.__API_SECRET_KEY = os.getenv("API_SECRET_KEY")
            except:
                raise Exception("API_SECRET_KEY not found in environment variables and not provided as an argument")
        else:
            self.__API_SECRET_KEY = api_secret_key
        
        if access_token is None:
            try:
                self.__ACCESS_TOKEN = os.getenv("ACCESS_TOKEN")
            except:
                raise Exception("ACCESS_TOKEN not found in environment variables and not provided as an argument")
        else:
            self.__ACCESS_TOKEN = access_token
        
        if access_token_secret is None:
            try:
                self.__ACCESS_TOKEN_SECRET = os.getenv("ACCESS_TOKEN_SECRET")
            except:
                raise Exception("ACCESS_TOKEN_SECRET not found in environment variables and not provided as an argument")
        else:
            self.__ACCESS_TOKEN_SECRET = access_token_secret
        
        if google_api_key is None:
            try:
                self.__GOOGLE_API_KEY = os.getenv("GOOGLE_API_KEY")
            except:
                pass
        else:
            self.__GOOGLE_API_KEY = google_api_key

        try:
            self.__client = Client(consumer_key=self.__API_KEY, 
                                   consumer_secret=self.__API_SECRET_KEY, 
                                   access_token=self.__ACCESS_TOKEN, 
                                   access_token_secret=self.__ACCESS_TOKEN_SECRET)
        except:
            raise Exception("Authentication Failed. Invalid Twitter API Credentials")
    


  #####################################
  # Private Methods
  #####################################

    def __tweet(self, text: str) -> dict:
        """Private: Tweet the given text
        
        Parameters
        ----------
        text : str
            The text to be tweeted

        Returns
        -------
        dict
            A dictionary containing the total character count, the number of tweets posted, and the id of the parent tweet
        """

        # Create chunks
        total_char_count = len(text)
        text = list(self.__create_chunks(text))


        no_of_chunks = len(text)

        # Seperate parent tweet from children tweets
        parent_tweet_text = text[0]
        leaf_tweets = text[1:]

        # Post parent tweet
        parent_tweet_id = self.__parent_tweet(text=parent_tweet_text)

        # Post children tweets
        for leaf_tweet in leaf_tweets:
            self.__child_tweet(text=leaf_tweet, parent_tweet_id=parent_tweet_id)

        return {"Total Character Count": total_char_count, "No. of Tweets": no_of_chunks, "Parent Tweet ID": parent_tweet_id}

    def __parent_tweet(self, text: str) -> str | None:
        """Post the parent tweet and return the id of the tweet
        
        Parameters
        ----------
        text : str
            The text to be tweeted
        
        Returns
        -------
        parent_tweet_id : str
            The id of the tweet
        """
        try:
            parent_tweet_id = self.__client.create_tweet(text=text).data['id']
            return parent_tweet_id
        except Exception as error:
            raise Exception(f"Tweet not posted succesfully: {error}")

    def __child_tweet(self, text: str, parent_tweet_id: str) -> None:
        """Post the child tweet as a reply to the parent tweet
        
        Parameters
        ----------
        text : str
            The text to be tweeted as a reply to the parent tweet
        parent_tweet_id : str
            The id of the parent tweet

        Returns
        -------
        None
        """
        try:
            self.__client.create_tweet(text=text, in_reply_to_tweet_id=parent_tweet_id)
        except Exception as error:
            raise Exception(f"Tweet not posted succesfully: {error}")


    def __create_chunks(self, text: str) -> list:
        """Create chunks of 280 characters each from the given text while leveraging the yield keyword
        
        Parameters
        ----------
        text : str
            The text to be chunked

        Returns
        -------
        chunks : list(str)
            A list of 280 char chunks of the given text
        """
        for start in range(0, len(text), 280):
            yield text[start:start + 280]

    
    def __summarize_article(self, article: dict, prompt: str|None) -> str | None:
        """Summarize the given article using Google PaLM API

        Parameters
        ----------
        article : dict
            A dictionary containing the title and article body of the news article
        prompt : str, optional
            The prompt to be used for the summarization, by default None

        Returns
        -------
        summary : str
            The summary of the article created using GOOGLE PaLM
        """

        # Get the API key
        if self.__GOOGLE_API_KEY is None:
            raise Exception("GOOGLE_API_KEY not found in environment variables")

        # Initialize PaLM
        try:
            palm.configure(api_key=self.__GOOGLE_API_KEY)
        except:
            raise Exception("Authentication Failed. Invalid Google API Credentials")

        # Default Settings
        defaults = {
                        'model': 'models/text-bison-001',
                        'temperature': 0.1,
                        'candidate_count': 1,
                        'top_k': 40,
                        'top_p': 0.95,
                        'max_output_tokens': 1024,
                        'stop_sequences': [],
                        'safety_settings': [{"category":"HARM_CATEGORY_DEROGATORY","threshold":1},{"category":"HARM_CATEGORY_TOXICITY","threshold":1},{"category":"HARM_CATEGORY_VIOLENCE","threshold":2},{"category":"HARM_CATEGORY_SEXUAL","threshold":2},{"category":"HARM_CATEGORY_MEDICAL","threshold":2},{"category":"HARM_CATEGORY_DANGEROUS","threshold":2}],
                    }
        
        # Create a prompt
        prompt = f"""
        Summarize the following article and condense it into 2 bullet points. Add the title of the article at the top. Do not leave an empty line after the article. Only use information from the article provided below. Structure your response as follows:

        Format for Summary:
        Title
        - Bullet point 1
        - Bullet point 2

        Title - {article['title']}
        Article - {article['article']}

        Summary:
        """ if prompt is None else f"""{prompt}"""

        # Generate the tweet
        try:
            tweet = palm.generate_text(**defaults, prompt=prompt)
            return tweet.result
        except:
            raise Exception("Failed to generate tweet using PaLM")
            

    def __clean_tweet(self, tweet_text: str) -> str:
        """Clean the tweet by removing unwanted characters as PaLM adds '*' to the tweet occasionally

        Parameters
        ----------
        tweet_text : str
            The text of the tweet to be cleaned

        Returns
        -------
        tweet_text : str
            The cleaned tweet text
        """

        # Remove * from tweet
        tweet_text = tweet_text.replace('*', '')
        return tweet_text

    def __handle_articles_list(self, articles_list: list, title: str|None, prompt: str|None) -> str | None:
        """Handle the list of articles by summarizing them and returning a generated tweet

        Parameters
        ----------
        articles_list : list(dict)
            A list of dictionaries containing the title, source, date and link of the articles
        title : str, optional
            The title of the tweet
        prompt : str, optional
            The prompt to be used for the summarization, by default None

        Returns
        -------
        articles_generated_summary : str
            The generated tweet from the articles
        """

        # Add title of tweet to the beginning of the tweet
        articles_generated_summary = f"{title}:\n\n"

        # Add the summary for each article to the tweet
        for article in articles_list:
            # Call API to get summary
            summary = self.__summarize_article(article, prompt)

            # If summary is None, continue by skipping article
            if summary is None:
                continue

            # Clean the summary
            articles_generated_summary += self.__clean_tweet(summary)

            # Add a new line, formatting
            articles_generated_summary += "\n\n"

        # Return the generated summary
        return articles_generated_summary
    
    def __generate_with_palm(self, prompt: str|None) -> str | None:
        """Generate a tweet using PaLM

        Parameters
        ----------
        prompt : str, optional
            The prompt to be used for generation, by default None
        to_rewrite : str, optional
            The text to be rewritten, by default None
        Returns
        -------
        generated_summary : str
            The generated tweet
        """

        # Get the API key
        try:
            GOOGLE_API_KEY = os.environ['GOOGLE_API_KEY']
        except:
            raise Exception("GOOGLE_API_KEY not found in environment variables")

        # Initialize PaLM
        try:
            palm.configure(api_key=GOOGLE_API_KEY)
        except:
            raise Exception("Authentication Failed. Invalid Google API Credentials")
        
        # Default Settings
        defaults = {
                        'model': 'models/text-bison-001',
                        'temperature': 0.1,
                        'candidate_count': 1,
                        'top_k': 40,
                        'top_p': 0.95,
                        'max_output_tokens': 1024,
                        'stop_sequences': [],
                        'safety_settings': [{"category":"HARM_CATEGORY_DEROGATORY","threshold":1},{"category":"HARM_CATEGORY_TOXICITY","threshold":1},{"category":"HARM_CATEGORY_VIOLENCE","threshold":2},{"category":"HARM_CATEGORY_SEXUAL","threshold":2},{"category":"HARM_CATEGORY_MEDICAL","threshold":2},{"category":"HARM_CATEGORY_DANGEROUS","threshold":2}],
                    }
        
        # Generate the tweet
        try:
            tweet = palm.generate_text(**defaults, prompt=prompt)
            return tweet.result
        except:
            raise Exception("Failed to generate tweet using PaLM")
        

        

  #####################################
  # Public Methods - API Methods
  #####################################

    def get_client(self) -> Client:
        """Get the tweepy client object

        Returns
        -------
        client : Client
            The tweepy client object

        Examples
        --------
        >>> from twitternewsbot.tweeter import Tweeter
        >>> tweeter = Tweeter()
        >>> client = tweeter.get_client() # Retrieve your tweety client object
        >>> client.create_tweet(text="Hello World!") # Tweet Hello World using your account's client
        """
        return self.__client

    def tweet(self, title: str|None = None, tweet: str|None = None, articles_list: list|None = None, use_palm: bool = False, prompt: str|None = None) -> dict:
        """Tweet the given articles list

        Parameters
        ----------
        title : str, optional
            The title of the tweet
        tweet : str, optional
            The tweet to be posted
        articles_list : list, optional
            A list of dictionaries containing the title, source, link and text of the articles.
            Obtained from the NewsFinder.get_news_articles() method
        use_palm : bool, optional
            A boolean value indicating whether to use PaLM to generate tweet with prompt
        prompt : str, optional
            The prompt to be used for the summarization with PaLM

        Returns
        -------
        tweet : dict
            A dictionary containing the total character count, the number of tweets posted, and the id of the parent tweet
        
        Raises
        ------
        TypeError
            If title is not a string
        TypeError
            If tweet is not a string
        TypeError
            If articles_list is not a list of dictionaries
        TypeError
            If use_palm is not a boolean
        ValueError
            If use_palm is True and tweet or articles_list is not None
        ValueError
            If use_palm is True and prompt is None
        ValueError
            If tweet and articles_list are both not None
        TypeError
            If prompt is not a string

        Examples
        --------

        Provide a title and tweet to be posted:

        >>> from twitternewsbot.tweeter import Tweeter
        >>> tweeter = Tweeter()
        >>> tweeter.tweet(title="Hello World", tweet="Hello World!")

        Provide a title and articles list to be posted:

        >>> from twitternewsbot.tweeter import Tweeter
        >>> from twitternewsbot.newsfinder import NewsFinder
        >>> tweeter = Tweeter()
        >>> newsfinder = NewsFinder()
        >>> articles_list = newsfinder.get_news_articles(topic="Bitcoin", num_articles=5, article_text=True) # Article Text must be True if summarizing articles
        >>> tweeter.tweet(title="Bitcoin News", articles_list=articles_list)

        Provide a prompt to be used with PaLM to generate a tweet:

        >>> from twitternewsbot.tweeter import Tweeter
        >>> tweeter = Tweeter()
        >>> tweeter.tweet(title="AI", use_palm=True, prompt="Create a 50 word description of what Artificial Intelligence is.")
        """

        # Check if title is valid
        if title is not None and not isinstance(title, str):
            raise TypeError("title must be a string")
        
        # Check if tweet is valid
        if tweet is not None and not isinstance(tweet, str):
            raise TypeError("tweet must be a string")
        
        # Check if articles_list is valid
        if articles_list is not None and (not isinstance(articles_list, list) or not all(isinstance(article, dict) for article in articles_list)):
            raise TypeError("articles_list must be a list of dicts")
        
        # Check if use_palm is valid
        if use_palm is not None and not isinstance(use_palm, bool):
            raise TypeError("use_palm must be a boolean")
        
        # Check if use_palm is True, then tweet and articles_list must be None
        if use_palm is True and (tweet is not None or articles_list is not None):
            raise ValueError("tweet and articles_list must be None if use_palm is True")
        
        # Check if prompt is not provided and use_palm is True
        if prompt is None and use_palm is True:
            raise ValueError("prompt must be provided if use_palm is True")

        # Check if tweet and articles_list are both provided
        if tweet is not None and articles_list is not None:
            raise ValueError("Both tweet and articles_list cannot be provided")
        
        # Check if prompt is valid
        if prompt is not None and not isinstance(prompt, str):
            raise TypeError("prompt must be a string")
        
        if articles_list is not None:
            # Handle the articles list
            tweet = self.__handle_articles_list(articles_list, title, prompt)
        elif use_palm:
            # Generate a tweet using PaLM
            tweet = self.__generate_with_palm(prompt)
            if title is not None:
                tweet = f"{title}:\n\n{tweet}"
        
        return self.__tweet(tweet)
        

        
        
        



from distutils.core import setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
  name = 'twitternewsbot',         # How you named your package folder (MyLib)
  packages = ['twitternewsbot'],   # Chose the same as "name"
  version = '2.0.3',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Python API allowing you to automize a personalized news delivery system.',   # Give a short description about your library
  author = 'Arnav Marda',                   # Type in your name
  author_email = 'arnavmarda@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/arnavmarda/twitter-news-bot/',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/arnavmarda/twitter-news-bot/archive/refs/tags/v2.0.3.tar.gz',    # I explain this later on
  keywords = ['AI', 'Twitter', 'News', 'Automation', 'Google News', 'Scraping', 'Requests', 'Tweepy', 'Articles'],   # Keywords that define your package best
  long_description = long_description,
  long_description_content_type="text/markdown",
  install_requires=[            # I get to this in a second
          'requests==2.31.0',
          'requests-html==0.10.0',
          'tweepy==4.14.0',
          'google-generativeai==0.1.0',
          'validators==0.20.0',
          'cron-validator==1.0.8',
          'python-dotenv==1.0.0'
      ],
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.11',
  ],
)